@@ -11,12 +11,14 @@ import subprocess
 import sys
 import time as tm
 import platform
+import uuid
 import requests
-import randomname
 
 from .worker import Worker
 from .simvue import Simvue
-from .utilities import get_auth
+from .models import RunInput
+from .utilities import get_auth, get_expiry
+from pydantic import ValidationError
 
 INIT_MISSING = 'initialize a run using init() first'
 QUEUE_SIZE = 10000
@@ -127,6 +129,7 @@ class Run(object):
     Track simulation details based on token and URL
     """
     def __init__(self, mode='online'):
+        self._uuid = str(uuid.uuid4())
         self._mode = mode
         self._name = None
         self._suppress_errors = False
@@ -144,6 +147,8 @@ class Run(object):
         self._url, self._token = get_auth()
         self._headers = {"Authorization": f"Bearer {self._token}"}
         self._simvue = None
+        self._pid = 0
+        self._resources_metrics_interval = 30
 
     def __enter__(self):
         return self
@@ -152,6 +157,13 @@ class Run(object):
         if self._name and self._status == 'running':
             self.set_status('completed')
 
+    def _check_token(self):
+        """
+        Check if token is valid
+        """
+        if self._mode == 'online' and tm.time() - get_expiry(self._token) > 0:
+            self._error('token has expired or is invalid')
+
     def _start(self, reconnect=False):
         """
         Start a run
@@ -159,6 +171,8 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
+        self._check_token()
+
         data = {'name': self._name, 'status': self._status}
         if reconnect:
             data['system'] = get_system()
@@ -168,9 +182,20 @@ class Run(object):
 
         self._start_time = tm.time()
 
+        if self._pid == 0:
+            self._pid = os.getpid()
+
         self._metrics_queue = multiprocessing.Manager().Queue(maxsize=self._queue_size)
         self._events_queue = multiprocessing.Manager().Queue(maxsize=self._queue_size)
-        self._worker = Worker(self._metrics_queue, self._events_queue, self._name, self._url, self._headers, self._mode)
+        self._worker = Worker(self._metrics_queue,
+                              self._events_queue,
+                              self._uuid,
+                              self._name,
+                              self._url,
+                              self._headers,
+                              self._mode,
+                              self._pid,
+                              self._resources_metrics_interval)
 
         if multiprocessing.current_process()._parent_pid is None:
             self._worker.start()
@@ -196,20 +221,12 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not name:
-            name = randomname.get_name()
-
         if not self._token or not self._url:
             self._error('Unable to get URL and token from environment variables or config file')
 
-        if not re.match(r'^[a-zA-Z0-9\-\_\s\/\.:]+$', name):
-            self._error('specified name is invalid')
-
-        if not isinstance(tags, list):
-            self._error('tags must be a list')
-
-        if not isinstance(metadata, dict):
-            self._error('metadata must be a dict')
+        if name:
+            if not re.match(r'^[a-zA-Z0-9\-\_\s\/\.:]+$', name):
+                self._error('specified name is invalid')
 
         self._name = name
 
@@ -218,32 +235,42 @@ class Run(object):
         else:
             self._status = 'created'
 
-        data = {'name': name,
-                'metadata': metadata,
+        data = {'metadata': metadata,
                 'tags': tags,
                 'system': {'cpu': {},
                            'gpu': {},
                            'platform': {}},
                 'status': self._status}
 
+        if name:
+            data['name'] = name
+
         if description:
             data['description'] = description
 
-        if not folder.startswith('/'):
-            self._error('the folder must begin with /')
-
         data['folder'] = folder
 
         if self._status == 'running':
             data['system'] = get_system()
 
-        self._simvue = Simvue(self._name, self._mode, self._suppress_errors)
-        if not self._simvue.create_run(data):
+        self._check_token()
+
+        # compare with pydantic RunInput model    
+        try:
+            runinput = RunInput(**data)
+        except ValidationError as e:
+            self._error(e)
+
+        self._simvue = Simvue(self._name, self._uuid, self._mode, self._suppress_errors)
+        name = self._simvue.create_run(data)
+
+        if not name:
             return False
+        elif name is not True:
+            self._name = name
 
         if self._status == 'running':
             self._start()
-
         return True
 
     @property
@@ -253,7 +280,14 @@ class Run(object):
         """
         return self._name
 
-    def reconnect(self, name):
+    @property
+    def uid(self):
+        """
+        Return the local unique identifier of the run
+        """
+        return self._uuid
+
+    def reconnect(self, name=None, uid=None):
         """
         Reconnect to a run in the created state
         """
@@ -262,13 +296,23 @@ class Run(object):
 
         self._status = 'running'
         self._name = name
-        self._simvue = Simvue(self._name, self._mode, self._suppress_errors)
+        self._uuid = uid
+
+        self._simvue = Simvue(self._name, self._uuid, self._mode, self._suppress_errors)
         self._start(reconnect=True)
 
+    def set_pid(self, pid):
+        """
+        Set pid of process to be monitored
+        """
+        self._pid = pid
+
     def config(self,
                suppress_errors=False,
                queue_blocking=False,
-               queue_size=QUEUE_SIZE):
+               queue_size=QUEUE_SIZE,
+               disable_resources_metrics=False,
+               resources_metrics_interval=30):
         """
         Optional configuration
         """
@@ -284,6 +328,16 @@ class Run(object):
             self._error('queue_size must be an integer')
         self._queue_size = queue_size
 
+        if not isinstance(disable_resources_metrics, bool):
+            self._error('disable_resources_metrics must be boolean')
+
+        if disable_resources_metrics:
+            self._pid = None
+
+        if not isinstance(resources_metrics_interval, int):
+            self._error('resources_metrics_interval must be an integer')
+        self._resources_metrics_interval = resources_metrics_interval
+
     def update_metadata(self, metadata):
         """
         Add/update metadata
@@ -291,7 +345,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -317,7 +371,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -339,7 +393,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -376,7 +430,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -427,7 +481,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -486,7 +540,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -536,7 +590,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -562,7 +616,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -579,7 +633,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -629,7 +683,7 @@ class Run(object):
         if self._mode == 'disabled':
             return True
 
-        if not self._name:
+        if not self._uuid and not self._name:
             self._error(INIT_MISSING)
             return False
 
@@ -642,11 +696,11 @@ class Run(object):
                 self._error('alert rule invalid')
                 return False
 
-        if type in ('is below', 'is above') and threshold is None:
+        if rule in ('is below', 'is above') and threshold is None:
             self._error('threshold must be defined for the specified alert type')
             return False
 
-        if type in ('is outside range', 'is inside range') and (range_low is None or range_high is None):
+        if rule in ('is outside range', 'is inside range') and (range_low is None or range_high is None):
             self._error('range_low and range_high must be defined for the specified alert type')
             return False
 
